'use babel';
'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _stringArgv = require('string-argv');

var _stringArgv2 = _interopRequireDefault(_stringArgv);

var _character = require('../../database/character');

var _character2 = _interopRequireDefault(_character);

var _commandFormat = require('../../util/errors/command-format');

var _commandFormat2 = _interopRequireDefault(_commandFormat);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

Function.prototype.$asyncbind = function $asyncbind(self, catcher) {
	var resolver = this;

	if (catcher === true) {
		if (!Function.prototype.$asyncbind.EagerThenable) Function.prototype.$asyncbind.EagerThenable = function factory(tick) {
			var _tasks = [];

			if (!tick) {
				try {
					tick = process.nextTick;
				} catch (ex) {
					tick = function tick(p) {
						setTimeout(p, 0);
					};
				}
			}

			function _untask() {
				for (var i = 0; i < _tasks.length; i += 2) {
					var t = _tasks[i + 1],
					    r = _tasks[i];

					for (var j = 0; j < t.length; j++) t[j].call(null, r);
				}

				_tasks = [];
			}

			function isThenable(obj) {
				return obj && obj instanceof Object && typeof obj.then === 'function';
			}

			function EagerThenable(resolver) {
				function done(inline) {
					var w;
					if (_sync || phase < 0 || (w = _thens[phase]).length === 0) return;

					_tasks.push(result, w);

					_thens = [[], []];
					if (_tasks.length === 2) inline ? _untask() : tick(_untask);
				}

				function resolveThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 0;
					result = x;
					done(true);
				}

				function rejectThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 1;
					result = x;
					done(true);
				}

				function settler(resolver, rejecter) {
					_thens[0].push(resolver);

					_thens[1].push(rejecter);

					done();
				}

				function toString() {
					return 'EagerThenable{' + {
						'-1': 'pending',
						0: 'resolved',
						1: 'rejected'
					}[phase] + '}=' + result.toString();
				}

				this.then = settler;
				this.toString = toString;
				var _thens = [[], []],
				    _sync = true,
				    phase = -1,
				    result;
				resolver.call(null, resolveThen, rejectThen);
				_sync = false;
				done();
			}

			EagerThenable.resolve = function (v) {
				return isThenable(v) ? v : {
					then: function then(resolve, reject) {
						return resolve(v);
					}
				};
			};

			return EagerThenable;
		}();
		return new Function.prototype.$asyncbind.EagerThenable(boundThen);
	}

	if (catcher) {
		if (Function.prototype.$asyncbind.wrapAsyncStack) catcher = Function.prototype.$asyncbind.wrapAsyncStack(catcher);
		return then;
	}

	function then(result, error) {
		try {
			return result && result instanceof Object && typeof result.then === 'function' ? result.then(then, catcher) : resolver.call(self, result, error || catcher);
		} catch (ex) {
			return (error || catcher)(ex);
		}
	}

	function boundThen(result, error) {
		return resolver.call(self, result, error);
	}

	boundThen.then = boundThen;
	return boundThen;
};

var newlinesPattern, newlinesReplacement, newlinesReplacementPattern, extraNewlinesPattern, mentionsPattern;
newlinesPattern = /\n/g;
newlinesReplacement = '{!~NL~!}';
newlinesReplacementPattern = new RegExp(newlinesReplacement, 'g');
extraNewlinesPattern = /\n{3,}/g;
mentionsPattern = /@everyone|@here|<@!?&?[0-9]+>/i;
exports.default = {
	name: 'addcharacter',
	aliases: ['addchar'],
	group: 'characters',
	groupName: 'add',
	description: 'Adds a character to the database, or updates the existing one.',
	usage: 'addcharacter <name> <info>',
	details: 'The character name can be a maximum of 60 characters long, and must be surrounded by quotes if it contains spaces. The information doesn\'t have to be a single line. Only the owner of the character and administrators/moderators may update it.',
	examples: ['addcharacter Bob Just your average guy.', 'addcharacter "Billy McBillface" A really cool guy who enjoys his chicken tendies.'],
	singleArgument: true,

	isRunnable(message) {
		return !!message.server;
	},

	run(message, args) {
		return new Promise(function ($return, $error) {
			var newlinesReplaced, argv, name, info, result;

			if (!args[0]) return $error(new _commandFormat2.default(this, message.server));
			if (mentionsPattern.test(args[0])) {
				message.reply('Please do not use mentions in your character name or information.');
				return $return();
			}

			// Extract the name and info
			newlinesReplaced = args[0].replace(newlinesPattern, newlinesReplacement);
			argv = (0, _stringArgv2.default)(newlinesReplaced);
			name = argv.shift().trim();
			info = argv.join(' ').replace(newlinesReplacementPattern, '\n').replace(extraNewlinesPattern, '\n\n');


			// Apply some restrictions
			if (!info) return $error(new _commandFormat2.default(this, message.server));
			if (name.length > 60) {
				message.reply('A character\'s name may not be longer than 60 characters.');
				return $return();
			}
			if (name.includes('\n')) {
				message.reply('A character\'s name may not have multiple lines.');
				return $return();
			}

			// Add or update the character
			return _character2.default.save(new _character2.default(message.server, message.author, name, info)).then(function ($await_1) {
				result = $await_1;

				if (result) {
					message.reply(`${ result.new ? 'Added' : 'Updated' } character "${ name }".`);
				} else {
					message.reply(`Unable to update character "${ name }". You are not the owner.`);
				}
				return $return();
			}.$asyncbind(this, $error), $error);
		}.$asyncbind(this));
	}
};