'use babel';
'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _localStorage = require('./local-storage');

var _localStorage2 = _interopRequireDefault(_localStorage);

var _search = require('../util/search');

var _search2 = _interopRequireDefault(_search);

var _logger = require('../util/logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

Function.prototype.$asyncbind = function $asyncbind(self, catcher) {
	var resolver = this;

	if (catcher === true) {
		if (!Function.prototype.$asyncbind.EagerThenable) Function.prototype.$asyncbind.EagerThenable = function factory(tick) {
			var _tasks = [];

			if (!tick) {
				try {
					tick = process.nextTick;
				} catch (ex) {
					tick = function tick(p) {
						setTimeout(p, 0);
					};
				}
			}

			function _untask() {
				for (var i = 0; i < _tasks.length; i += 2) {
					var t = _tasks[i + 1],
					    r = _tasks[i];

					for (var j = 0; j < t.length; j++) t[j].call(null, r);
				}

				_tasks = [];
			}

			function isThenable(obj) {
				return obj && obj instanceof Object && typeof obj.then === 'function';
			}

			function EagerThenable(resolver) {
				function done(inline) {
					var w;
					if (_sync || phase < 0 || (w = _thens[phase]).length === 0) return;

					_tasks.push(result, w);

					_thens = [[], []];
					if (_tasks.length === 2) inline ? _untask() : tick(_untask);
				}

				function resolveThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 0;
					result = x;
					done(true);
				}

				function rejectThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 1;
					result = x;
					done(true);
				}

				function settler(resolver, rejecter) {
					_thens[0].push(resolver);

					_thens[1].push(rejecter);

					done();
				}

				function toString() {
					return 'EagerThenable{' + {
						'-1': 'pending',
						0: 'resolved',
						1: 'rejected'
					}[phase] + '}=' + result.toString();
				}

				this.then = settler;
				this.toString = toString;
				var _thens = [[], []],
				    _sync = true,
				    phase = -1,
				    result;
				resolver.call(null, resolveThen, rejectThen);
				_sync = false;
				done();
			}

			EagerThenable.resolve = function (v) {
				return isThenable(v) ? v : {
					then: function then(resolve, reject) {
						return resolve(v);
					}
				};
			};

			return EagerThenable;
		}();
		return new Function.prototype.$asyncbind.EagerThenable(boundThen);
	}

	if (catcher) {
		if (Function.prototype.$asyncbind.wrapAsyncStack) catcher = Function.prototype.$asyncbind.wrapAsyncStack(catcher);
		return then;
	}

	function then(result, error) {
		try {
			return result && result instanceof Object && typeof result.then === 'function' ? result.then(then, catcher) : resolver.call(self, result, error || catcher);
		} catch (ex) {
			return (error || catcher)(ex);
		}
	}

	function boundThen(result, error) {
		return resolver.call(self, result, error);
	}

	boundThen.then = boundThen;
	return boundThen;
};

class ModRole {
	static loadDatabase() {
		this.serversMap = JSON.parse(_localStorage2.default.getItem('mod-roles'));
		if (!this.serversMap) this.serversMap = {};
	}

	static saveDatabase() {
		_logger2.default.debug('Saving mod roles storage...', this.serversMap);
		_localStorage2.default.setItem('mod-roles', JSON.stringify(this.serversMap));
	}

	static save(role) {
		if (!role) throw new Error('A role must be specified.');
		if (!this.serversMap) this.loadDatabase();
		if (!this.serversMap[role.server.id]) this.serversMap[role.server.id] = [];
		const serverRoles = this.serversMap[role.server.id];

		if (!serverRoles.includes(role.id)) {
			serverRoles.push(role.id);
			_logger2.default.info('Added new mod role.', this.basicInfo(role));
			this.saveDatabase();
			return true;
		} else {
			_logger2.default.info('Not adding mod role, because it already exists.', this.basicInfo(role));
			return false;
		}
	}

	static delete(role) {
		if (!role) throw new Error('A role must be specified.');
		if (!this.serversMap) this.loadDatabase();
		if (!this.serversMap[role.server.id]) return false;
		const serverRoles = this.serversMap[role.server.id];

		const roleIndex = serverRoles.findIndex(element => element === role.id);
		if (roleIndex >= 0) {
			serverRoles.splice(roleIndex, 1);
			_logger2.default.info('Deleted mod role.', this.basicInfo(role));
			this.saveDatabase();
			return true;
		} else {
			_logger2.default.info('Not deleting mod role, because it doesn\'t exist.', this.basicInfo(role));
			return false;
		}
	}

	static findInServer(server, searchString = null) {
		if (!server) throw new Error('A server must be specified.');
		if (!this.serversMap) this.loadDatabase();
		if (!this.serversMap[server.id]) return [];

		// Find all of the server's roles that match, and filter them to ones that are mod roles
		const roles = (0, _search2.default)(server.roles, searchString, { searchExact: false }).filter(r => this.serversMap[server.id].includes(r.id));
		return (0, _search2.default)(roles, searchString, { searchInexact: false });
	}

	static serverHasAny(server) {
		if (!server) throw new Error('A server must be specified.');
		if (!this.serversMap) this.loadDatabase();
		return this.serversMap[server.id] && this.serversMap[server.id].length > 0;
	}

	static basicInfo(role) {
		return { id: role.id, name: role.name, server: role.server.name, serverID: role.server.id };
	}
}
exports.default = ModRole;