'use babel';
'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _localStorage = require('./local-storage');

var _localStorage2 = _interopRequireDefault(_localStorage);

var _logger = require('../util/logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

Function.prototype.$asyncbind = function $asyncbind(self, catcher) {
	var resolver = this;

	if (catcher === true) {
		if (!Function.prototype.$asyncbind.EagerThenable) Function.prototype.$asyncbind.EagerThenable = function factory(tick) {
			var _tasks = [];

			if (!tick) {
				try {
					tick = process.nextTick;
				} catch (ex) {
					tick = function tick(p) {
						setTimeout(p, 0);
					};
				}
			}

			function _untask() {
				for (var i = 0; i < _tasks.length; i += 2) {
					var t = _tasks[i + 1],
					    r = _tasks[i];

					for (var j = 0; j < t.length; j++) t[j].call(null, r);
				}

				_tasks = [];
			}

			function isThenable(obj) {
				return obj && obj instanceof Object && typeof obj.then === 'function';
			}

			function EagerThenable(resolver) {
				function done(inline) {
					var w;
					if (_sync || phase < 0 || (w = _thens[phase]).length === 0) return;

					_tasks.push(result, w);

					_thens = [[], []];
					if (_tasks.length === 2) inline ? _untask() : tick(_untask);
				}

				function resolveThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 0;
					result = x;
					done(true);
				}

				function rejectThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 1;
					result = x;
					done(true);
				}

				function settler(resolver, rejecter) {
					_thens[0].push(resolver);

					_thens[1].push(rejecter);

					done();
				}

				function toString() {
					return 'EagerThenable{' + {
						'-1': 'pending',
						0: 'resolved',
						1: 'rejected'
					}[phase] + '}=' + result.toString();
				}

				this.then = settler;
				this.toString = toString;
				var _thens = [[], []],
				    _sync = true,
				    phase = -1,
				    result;
				resolver.call(null, resolveThen, rejectThen);
				_sync = false;
				done();
			}

			EagerThenable.resolve = function (v) {
				return isThenable(v) ? v : {
					then: function then(resolve, reject) {
						return resolve(v);
					}
				};
			};

			return EagerThenable;
		}();
		return new Function.prototype.$asyncbind.EagerThenable(boundThen);
	}

	if (catcher) {
		if (Function.prototype.$asyncbind.wrapAsyncStack) catcher = Function.prototype.$asyncbind.wrapAsyncStack(catcher);
		return then;
	}

	function then(result, error) {
		try {
			return result && result instanceof Object && typeof result.then === 'function' ? result.then(then, catcher) : resolver.call(self, result, error || catcher);
		} catch (ex) {
			return (error || catcher)(ex);
		}
	}

	function boundThen(result, error) {
		return resolver.call(self, result, error);
	}

	boundThen.then = boundThen;
	return boundThen;
};

class Setting {
	constructor(server, key, value) {
		if (!key) throw new Error('Setting key must be specified.');
		this.key = key;
		this.value = value;
		this.server = server ? server.id ? server.id : server : 'global';
	}

	static loadDatabase() {
		this.serversMap = JSON.parse(_localStorage2.default.getItem('settings'));
		if (!this.serversMap) this.serversMap = {};
	}

	static saveDatabase() {
		_logger2.default.debug('Saving settings storage...', this.serversMap);
		_localStorage2.default.setItem('settings', JSON.stringify(this.serversMap));
	}

	static save(setting) {
		if (!setting) throw new Error('A setting must be specified.');
		if (!this.serversMap) this.loadDatabase();
		if (!this.serversMap[setting.server]) this.serversMap[setting.server] = {};
		this.serversMap[setting.server][setting.key] = setting.value;
		_logger2.default.info('Saved setting.', setting);
		this.saveDatabase();
		return true;
	}

	static delete(setting, server = null) {
		[setting, server] = this.getSettingKeyAndServer(setting, server);
		if (!this.serversMap) this.loadDatabase();
		if (!this.serversMap[server]) return false;
		if (typeof this.serversMap[server][setting] === 'undefined') return false;
		delete this.serversMap[server][setting];
		_logger2.default.info('Deleted setting.', { key: setting, server: server });
		this.saveDatabase();
		return true;
	}

	static get(setting, server = null) {
		[setting, server] = this.getSettingKeyAndServer(setting, server);
		if (!this.serversMap) this.loadDatabase();
		if (!this.serversMap[server]) return null;
		return new Setting(server, setting, this.serversMap[server][setting]);
	}

	static getValue(setting, defaultValue = null, server = null) {
		[setting, server] = this.getSettingKeyAndServer(setting, server);
		if (!this.serversMap) this.loadDatabase();
		if (!this.serversMap[server]) return defaultValue;
		return setting in this.serversMap[server] ? this.serversMap[server][setting] : defaultValue;
	}

	static getSettingKeyAndServer(setting, server) {
		if (setting instanceof Setting) {
			return [setting.key, !server ? setting.server : server.id ? server.id : server];
		} else {
			if (!setting) throw new Error('A setting or a key must be specified.');
			return [setting, server ? server.id ? server.id : server : 'global'];
		}
	}
}
exports.default = Setting;