#!/usr/bin/env node

'use babel';
'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.unprefixedCommandPattern = exports.serverCommandPatterns = exports.client = undefined;

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _discord = require('discord.js');

var _discord2 = _interopRequireDefault(_discord);

var _stringArgv = require('string-argv');

var _stringArgv2 = _interopRequireDefault(_stringArgv);

var _commonTags = require('common-tags');

var _version = require('./version');

var _version2 = _interopRequireDefault(_version);

var _commands = require('./commands');

var _commands2 = _interopRequireDefault(_commands);

var _database = require('./database');

var _logger = require('./util/logger');

var _logger2 = _interopRequireDefault(_logger);

var _commandPattern = require('./util/command-pattern');

var _commandPattern2 = _interopRequireDefault(_commandPattern);

var _updateCheck = require('./util/update-check');

var _updateCheck2 = _interopRequireDefault(_updateCheck);

var _commandUsage = require('./util/command-usage');

var usage = _interopRequireWildcard(_commandUsage);

var _analytics = require('./util/analytics');

var analytics = _interopRequireWildcard(_analytics);

var _friendly = require('./util/errors/friendly');

var _friendly2 = _interopRequireDefault(_friendly);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

Function.prototype.$asyncbind = function $asyncbind(self, catcher) {
	var resolver = this;

	if (catcher === true) {
		if (!Function.prototype.$asyncbind.EagerThenable) Function.prototype.$asyncbind.EagerThenable = function factory(tick) {
			var _tasks = [];

			if (!tick) {
				try {
					tick = process.nextTick;
				} catch (ex) {
					tick = function tick(p) {
						setTimeout(p, 0);
					};
				}
			}

			function _untask() {
				for (var i = 0; i < _tasks.length; i += 2) {
					var t = _tasks[i + 1],
					    r = _tasks[i];

					for (var j = 0; j < t.length; j++) t[j].call(null, r);
				}

				_tasks = [];
			}

			function isThenable(obj) {
				return obj && obj instanceof Object && typeof obj.then === 'function';
			}

			function EagerThenable(resolver) {
				function done(inline) {
					var w;
					if (_sync || phase < 0 || (w = _thens[phase]).length === 0) return;

					_tasks.push(result, w);

					_thens = [[], []];
					if (_tasks.length === 2) inline ? _untask() : tick(_untask);
				}

				function resolveThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 0;
					result = x;
					done(true);
				}

				function rejectThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 1;
					result = x;
					done(true);
				}

				function settler(resolver, rejecter) {
					_thens[0].push(resolver);

					_thens[1].push(rejecter);

					done();
				}

				function toString() {
					return 'EagerThenable{' + {
						'-1': 'pending',
						0: 'resolved',
						1: 'rejected'
					}[phase] + '}=' + result.toString();
				}

				this.then = settler;
				this.toString = toString;
				var _thens = [[], []],
				    _sync = true,
				    phase = -1,
				    result;
				resolver.call(null, resolveThen, rejectThen);
				_sync = false;
				done();
			}

			EagerThenable.resolve = function (v) {
				return isThenable(v) ? v : {
					then: function then(resolve, reject) {
						return resolve(v);
					}
				};
			};

			return EagerThenable;
		}();
		return new Function.prototype.$asyncbind.EagerThenable(boundThen);
	}

	if (catcher) {
		if (Function.prototype.$asyncbind.wrapAsyncStack) catcher = Function.prototype.$asyncbind.wrapAsyncStack(catcher);
		return then;
	}

	function then(result, error) {
		try {
			return result && result instanceof Object && typeof result.then === 'function' ? result.then(then, catcher) : resolver.call(self, result, error || catcher);
		} catch (ex) {
			return (error || catcher)(ex);
		}
	}

	function boundThen(result, error) {
		return resolver.call(self, result, error);
	}

	boundThen.then = boundThen;
	return boundThen;
};

_logger2.default.info('RPBot v' + _version2.default + ' is starting...');
analytics.sendEvent('Bot', 'started');

// Output safe config
const debugConfig = Object.assign({}, _config2.default);
if (debugConfig.email) debugConfig.email = '--snip--';
if (debugConfig.password) debugConfig.password = '--snip--';
if (debugConfig.token) debugConfig.token = '--snip--';
for (const key of Object.keys(debugConfig)) if (key.length === 1 || key.includes('-')) delete debugConfig[key];
_logger2.default.debug('Configuration:', debugConfig);

// Verify that the credentials are usable
if (!_config2.default.token && (!_config2.default.email || !_config2.default.password)) {
	_logger2.default.error('Invalid credentials; either "token" or both "email" and "password" must be specified.');
	process.exit(1);
}

// Set up database
(0, _database.init)().catch(err => {
	_logger2.default.error(err);
	process.exit(1);
});

// Create client
const clientOptions = { autoReconnect: _config2.default.autoReconnect, forceFetchUsers: true, disableEveryone: true };
const client = exports.client = new _discord2.default.Client(clientOptions);
_logger2.default.info('Client created.', clientOptions);
client.on('error', e => {
	_logger2.default.error(e);
});
client.on('warn', e => {
	_logger2.default.warn(e);
});
client.on('debug', e => {
	_logger2.default.debug(e);
});
client.on('disconnected', () => {
	_logger2.default.error('Disconnected.');
});
client.on('ready', () => {
	_logger2.default.info(`Bot is ready; logged in as ${ client.user.username }#${ client.user.discriminator } (ID: ${ client.user.id })`);
	(0, _updateCheck2.default)();
	if (_config2.default.updateCheck > 0) setInterval(_updateCheck2.default, _config2.default.updateCheck * 60 * 1000);
});

// Set up command recognition
const serverCommandPatterns = exports.serverCommandPatterns = {};
const unprefixedCommandPattern = exports.unprefixedCommandPattern = /^([^\s]+)/i;
client.on('message', message => {
	if (message.author.equals(client.user)) return;
	let runCommand;
	let runArgs;
	let runFromPattern = false;

	// Find the command to run by patterns
	commandLoop: for (const command of _commands2.default) {
		if (!command.patterns) continue;
		for (const pattern of command.patterns) {
			const matches = pattern.exec(message.content);
			if (matches) {
				runCommand = command;
				runArgs = matches;
				runFromPattern = true;
				break commandLoop;
			}
		}
	}

	// Find the command to run with default command handling
	const patternIndex = message.server ? message.server.id : '-';
	if (!serverCommandPatterns[patternIndex]) serverCommandPatterns[patternIndex] = (0, _commandPattern2.default)(message.server, client.user);
	let defaultMatches;
	let unprefixedMatches;
	if (!runCommand) {
		defaultMatches = serverCommandPatterns[patternIndex].exec(message.content);
		if (defaultMatches) {
			const commandName = defaultMatches[2].toLowerCase();
			const command = _commands2.default.find(command => command.name === commandName || command.aliases && command.aliases.some(alias => alias === commandName));
			if (command && !command.disableDefault) {
				const argString = message.content.substring(defaultMatches[1].length + defaultMatches[2].length);
				runCommand = command;
				runArgs = !command.singleArgument ? (0, _stringArgv2.default)(argString) : [argString.trim()];
			}
		} else if (!message.server) {
			unprefixedMatches = unprefixedCommandPattern.exec(message.content);
			if (unprefixedMatches) {
				const commandName = unprefixedMatches[1].toLowerCase();
				const command = _commands2.default.find(command => command.name === commandName || command.aliases && command.aliases.some(alias => alias === commandName));
				if (command && !command.disableDefault) {
					const argString = message.content.substring(unprefixedMatches[1].length);
					runCommand = command;
					runArgs = !command.singleArgument ? (0, _stringArgv2.default)(argString) : [argString.trim()];
				}
			}
		}
	}

	// Run the command
	if (runCommand) {
		const logInfo = {
			args: runArgs.toString(),
			user: message.author.username + '#' + message.author.discriminator,
			userID: message.author.id,
			server: message.server ? message.server.name : null,
			serverID: message.server ? message.server.id : null
		};

		if (runCommand.isRunnable(message)) {
			_logger2.default.info(`Running ${ runCommand.group }:${ runCommand.groupName }.`, logInfo);
			analytics.sendEvent('Command', 'run', runCommand.group + ':' + runCommand.groupName);
			runCommand.run(message, runArgs, runFromPattern).catch(err => {
				if (err instanceof _friendly2.default) {
					message.reply(err.message);
				} else {
					const owner = _config2.default.owner ? client.users.get('id', _config2.default.owner) : null;
					message.reply(_commonTags.stripIndents`
						An error occurred while running the command.
						\`${ err.name }: ${ err.message }\`
						${ owner ? `Please contact ${ owner.name }#${ owner.discriminator }${ _config2.default.invite ? ` in ${ _config2.default.invite }` : '' }.` : '' }
					`);
					_logger2.default.error(err);
					analytics.sendException(err);
				}
			});
		} else {
			message.reply(`The \`${ runCommand.name }\` command is not currently usable in your context.`);
			_logger2.default.info(`Not running ${ runCommand.group }:${ runCommand.groupName }; not runnable.`, logInfo);
		}
	} else if (defaultMatches || unprefixedMatches) {
		if (!_config2.default.unknownOnlyMention || defaultMatches[1].startsWith('<@')) message.reply(`Unknown command. Use ${ usage.long('help', message.server) } to view the list of all commands.`);
	}
});

// Log in
const loginCallback = e => {
	if (e) _logger2.default.error('Failed to login.', e);
};
if (_config2.default.token) {
	_logger2.default.info('Logging in with token...');
	client.loginWithToken(_config2.default.token, _config2.default.email, _config2.default.password, loginCallback);
} else {
	_logger2.default.info('Logging in with email and password...');
	client.login(_config2.default.email, _config2.default.password, loginCallback);
}

// Exit on interrupt
let interruptCount = 0;
process.on('SIGINT', () => {
	return new Promise(function ($return, $error) {
		interruptCount++;

		function $IfStatement_1() {
			return $return();
		}

		if (interruptCount === 1) {
			_logger2.default.info('Received interrupt signal; closing database, destroying client, and exiting...');
			return Promise.all([(0, _database.close)(), client.destroy()]).catch(err => {
				_logger2.default.error(err);
			}).then(function ($await_2) {
				process.exit(0);
				return $IfStatement_1.call(this);
			}.$asyncbind(this, $error), $error);
		} else {
			_logger2.default.info('Received another interrupt signal; immediately exiting.');
			process.exit(0);
			return $IfStatement_1.call(this);
		}
	}.$asyncbind(undefined));
});